import time

from pygame import mixer


class MusicPlayer:
    """
    This class interfaces a musicplayer
    """

    def __init__(self) -> None:
        mixer.init()
        self._is_paused = False
        self._maxvolume = 0.75
        self._volume = self._maxvolume
        self._volume_fadespeed = 0.02

    @property
    def is_playing(self) -> bool:
        return mixer.music.get_busy()

    @property
    def is_paused(self) -> bool:
        return self._is_paused

    def play_song(self, song) -> None:
        mixer.music.load(song)
        self._is_paused = False
        self._set_volume(self._maxvolume)
        mixer.music.play()

    def stop_music_if_playing(self) -> None:
        if mixer.music.get_busy():
            self._is_paused = False
            mixer.music.stop()

    def stop(self) -> None:
        self._is_paused = False
        if mixer.music.get_busy():
            mixer.music.stop()

    def fade_down_music(self, pause: bool) -> None:
        """
        Fades down the current song until stopped. If pause is True,
        it will resume the current song when starting again
        """
        while self._volume > 0:
            self._volume = self._volume - self._volume_fadespeed
            if self._volume < 0:
                self._volume = 0

            self._set_volume(self._volume)
            time.sleep(0.05)

        if pause:
            self._pause()
        else:
            self.stop()

    def fade_up_music(self) -> None:
        self._unpause()
        while self._volume < self._maxvolume:
            self._volume = self._volume + self._volume_fadespeed
            if self._volume > self._maxvolume:
                self._volume = self._maxvolume

            self._set_volume(self._volume)
            time.sleep(0.05)

    def _pause(self) -> None:
        self._is_paused = True
        mixer.music.pause()

    def _unpause(self) -> None:
        self._is_paused = False
        mixer.music.unpause()

    def _set_volume(self, volume: float) -> None:
        self._volume = volume
        mixer.music.set_volume(volume)



import argparse

import confuse

from bgm.music_player import MusicPlayer
from bgm.music_state_machine import MusicStateMachine
from bgm.process_service import ProcessService


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Parse arguments for debugging purposes"
    )
    parser.add_argument("-es", nargs="?", help="the process name for emulationstation")
    parser.add_argument(
        "-names",
        metavar="ENames",
        type=str,
        nargs="*",
        help="the emulator names you want to consider",
    )

    args = parser.parse_args()
    config = confuse.Configuration('esbgm', __name__)
    config.set_args(args, dots=True)
    print('configuration directory is', config.config_dir())
    MusicStateMachine(
        ProcessService(),
        MusicPlayer(),
        config
    ).run()


if __name__ == "__main__":
    main()



import os
import random
import time
from configparser import ConfigParser
from enum import Enum, auto
from queue import LifoQueue, Queue
from typing import List, Optional

from confuse import Configuration
from transitions import Machine

from bgm.music_player import MusicPlayer
from bgm.process_service import ProcessService


class MusicState(Enum):
    PAUSED = "paused"
    STOPPED = "stopped"
    PLAYING_MUSIC = "playing_music"


class MusicStateMachine:
    random.seed()

    class _EnvironmentState:
        def __init__(
            self,
            music_is_disabled: bool,
            es_running: bool,
            emulator_is_running: bool,
            song_is_being_played: bool,
            restart: bool,
        ) -> None:
            self.restart = restart
            self.song_is_being_played = song_is_being_played
            self.stopper_process_is_running = emulator_is_running
            self.main_process_running = es_running
            self.music_is_disabled = music_is_disabled

    def __init__(
        self,
        process_service: ProcessService,
        music_player: MusicPlayer,
        config: Configuration,
        forced_initial_status: Optional[MusicState] = None
    ):
        self._process_service = process_service
        self._music_player = music_player

        self._startdelay = config["startdelay"].get()
        self._musicdir = config["musicdir"].as_filename()
        self._restart = config["restart"].get()
        self._startsong = config["startsong"].get()

        self._song_queue = self._generate_random_music_queue()

        self._mainprocess = config["mainprocess"].get()
        self._stopper_processes = config["emulator_names"].get()

        self._machine = Machine(model=self, states=MusicState, initial=forced_initial_status or self._get_initial_state())
        self._machine.add_transition("run_trns", MusicState.PAUSED, MusicState.STOPPED, before="stop_music", conditions="has_to_stop_music")
        self._machine.add_transition("run_trns", MusicState.PAUSED, MusicState.PLAYING_MUSIC, before="fade_up", conditions="stopper_process_is_not_running")
        self._machine.add_transition("run_trns", MusicState.STOPPED, MusicState.PLAYING_MUSIC, before="play_music", conditions="has_to_play_music")
        self._machine.add_transition("run_trns", MusicState.PLAYING_MUSIC, MusicState.STOPPED, before="stop_music",
                                     conditions="has_to_stop_music")
        self._machine.add_transition("run_trns", MusicState.PLAYING_MUSIC, MusicState.PAUSED, before="fade_down",
                                     conditions="should_fade_down_and_pause")
        self._machine.add_transition("run_trns", MusicState.PLAYING_MUSIC, MusicState.STOPPED, before="fade_down",
                                     conditions="should_fade_down_and_stop")
        self._machine.add_transition("run_trns", MusicState.PLAYING_MUSIC, MusicState.PLAYING_MUSIC, before="play_music",
                                     conditions="_music_is_not_playing")

    def execute_state(self) -> None:
        state = self._get_state()
        self.run_trns(state)

    def run(self) -> None:
        if self._startdelay > 0:
            time.sleep(self._startdelay)

        self._wait_splash_screen()

        while True:
            self.execute_state()
            time.sleep(2)

    def _get_songs(self) -> List[str]:
        return [
            song
            for song in os.listdir(self._musicdir)
            if song[-4:] == ".mp3" or song[-4:] == ".ogg"
        ]

    def _generate_random_music_queue(self) -> Queue:
        song_queue: Queue[str] = LifoQueue()
        song_list = self._get_songs()

        if self._startsong in song_list:
            song_queue.put(self._startsong)
            song_list.remove(self._startsong)

        while len(song_list) > 0:
            song_to_add = random.randint(0, len(song_list) - 1)
            song_queue.put(song_list[song_to_add])
            song_list.remove(song_list[song_to_add])

        return song_queue

    # ACTIONS

    def delay(self) -> None:
        time.sleep(2)

    def stop_music(self, status: _EnvironmentState) -> None:
        self._music_player.stop()

    def fade_up(self, status: _EnvironmentState) -> None:
        self._music_player.fade_up_music()

    def fade_down(self, status: _EnvironmentState) -> None:
         self._music_player.fade_down_music(not self._restart)

    def play_music(self, status: _EnvironmentState) -> None:
        if self._song_queue.empty():
            self._song_queue = self._generate_random_music_queue()

        song = os.path.join(self._musicdir, self._song_queue.get())
        self._music_player.play_song(song)

    # CONDITIONS

    def has_to_stop_music(self, status: _EnvironmentState) -> bool:
        return not status.stopper_process_is_running and (
            status.music_is_disabled or not status.main_process_running
        )

    def stopper_process_is_not_running(self, status: _EnvironmentState) -> bool:
        return not status.stopper_process_is_running

    def should_fade_down_and_stop(self, status: _EnvironmentState) -> bool:
        return status.stopper_process_is_running and self._restart

    def should_fade_down_and_pause(self, status: _EnvironmentState) -> bool:
        return status.stopper_process_is_running and not self._restart

    def has_to_play_music(self, status: _EnvironmentState) -> bool:
        return status.main_process_running and not status.stopper_process_is_running

    def _music_is_not_playing(self, status: _EnvironmentState) -> bool:
        return not self._music_player.is_playing

    def _get_initial_state(self) -> MusicState:
        if self._music_player.is_playing:
            return MusicState.PLAYING_MUSIC
        elif self._music_player.is_paused:
            return MusicState.PAUSED
        else:
            return MusicState.STOPPED

    def _get_state(self) -> _EnvironmentState:
        status = self._EnvironmentState(
            emulator_is_running=self._process_service.any_process_is_running(
                self._stopper_processes
            ),
            es_running=self._process_service.process_is_running(
                self._mainprocess
            ),
            music_is_disabled=self._music_is_disabled(),
            restart=self._restart,
            song_is_being_played=self._music_player.is_playing,
        )

        return status

    def _music_is_disabled(self) -> bool:
        return os.path.exists("/home/pi/PyScripts/DisableMusic")

    def _wait_splash_screen(self) -> None:
        # Look for OMXplayer - if it's running, someone's got a splash screen going!
        while self._process_service.process_is_running("omxplayer") or self._process_service.process_is_running("omxplayer.bin"):
            time.sleep(
                1
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bgm']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'confuse>=1.7.0,<2.0.0',
 'deepmerge>=1.0.1,<2.0.0',
 'psutil>=5.9.0,<6.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'pygame>=2.0.1,<3.0.0',
 'python-decouple>=3.4,<4.0',
 'transitions>=0.8.10,<0.9.0']

extras_require = \
{'lint': ['pylint']}

setup_kwargs = {
    'name': 'es-bgm',
    'version': '1.0.1',
    'description': 'Allows you to add background music to EmulationStation',
    'long_description': None,
    'author': 'David',
    'author_email': 'davigetto@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
